// GENERATED BY MITOSIS

import { getDefaultRegisteredComponents } from "../../constants/builder-registered-components.js";
import { TARGET } from "../../constants/target.js";
import BuilderContext from "../../context/builder.context";
import { evaluate } from "../../functions/evaluate.js";
import { getContent } from "../../functions/get-content/index.js";
import { getFetch } from "../../functions/get-fetch.js";
import { isBrowser } from "../../functions/is-browser.js";
import { isEditing } from "../../functions/is-editing.js";
import { isPreviewing } from "../../functions/is-previewing.js";
import { markMutable } from "../../functions/mark-mutable.js";
import {
  components,
  createRegisterComponentMessage,
} from "../../functions/register-component.js";
import { track } from "../../functions/track.js";
import {
  registerInsertMenu,
  setupBrowserForEditing,
} from "../../scripts/init-editing.js";
import RenderBlocks from "../render-blocks";
import RenderContentStyles from "./components/render-styles";
import {
  Fragment,
  _useMutableProps,
  component$,
  h,
  useCleanup$,
  useClientEffect$,
  useContextProvider,
  useRef,
  useStore,
  useWatch$,
} from "@builder.io/qwik";
export const useContent = function useContent(props, state, elementRef) {
  const mergedContent = {
    ...props.content,
    ...state.overrideContent,
    data: {
      ...props.content?.data,
      ...props.data,
      ...state.overrideContent?.data,
    },
  };
  return mergedContent;
};
export const canTrackToUse = function canTrackToUse(props, state, elementRef) {
  return props.canTrack || true;
};
export const contentState = function contentState(props, state, elementRef) {
  return {
    ...props.content?.data?.state,
    ...props.data,
    ...state.overrideState,
  };
};
export const contextContext = function contextContext(
  props,
  state,
  elementRef
) {
  return props.context || {};
};
export const allRegisteredComponents = function allRegisteredComponents(
  props,
  state,
  elementRef
) {
  const allComponentsArray = [
    ...getDefaultRegisteredComponents(), // While this `components` object is deprecated, we must maintain support for it.
    // Since users are able to override our default components, we need to make sure that we do not break such
    // existing usage.
    // This is why we spread `components` after the default Builder.io components, but before the `props.customComponents`,
    // which is the new standard way of providing custom components, and must therefore take precedence.
    ...components,
    ...(props.customComponents || []),
  ];
  const allComponents = allComponentsArray.reduce(
    (acc, curr) => ({ ...acc, [curr.name]: curr }),
    {}
  );
  return allComponents;
};
export const processMessage = function processMessage(
  props,
  state,
  elementRef,
  event
) {
  const { data } = event;

  if (data) {
    switch (data.type) {
      case "builder.contentUpdate": {
        const messageContent = data.data;
        const key =
          messageContent.key ||
          messageContent.alias ||
          messageContent.entry ||
          messageContent.modelName;
        const contentData = messageContent.data;

        if (key === props.model) {
          state.overrideContent = contentData;
          state.forceReRenderCount = state.forceReRenderCount + 1; // This is a hack to force Qwik to re-render.
        }

        break;
      }

      case "builder.patchUpdates": {
        // TODO
        break;
      }
    }
  }
};
export const evaluateJsCode = function evaluateJsCode(
  props,
  state,
  elementRef
) {
  // run any dynamic JS code attached to content
  const jsCode = useContent(props, state, elementRef)?.data?.jsCode;

  if (jsCode) {
    evaluate({
      code: jsCode,
      context: contextContext(props, state, elementRef),
      state: contentState(props, state, elementRef),
    });
  }
};
export const httpReqsData = function httpReqsData(props, state, elementRef) {
  return {};
};
export const onClick = function onClick(props, state, elementRef, _event) {
  if (useContent(props, state, elementRef)) {
    track({
      type: "click",
      canTrack: canTrackToUse(props, state, elementRef),
      contentId: useContent(props, state, elementRef).id,
      orgId: props.apiKey,
    });
  }
};
export const evalExpression = function evalExpression(
  props,
  state,
  elementRef,
  expression
) {
  return expression.replace(/{{([^}]+)}}/g, (_match, group) =>
    evaluate({
      code: group,
      context: contextContext(props, state, elementRef),
      state: contentState(props, state, elementRef),
    })
  );
};
export const handleRequest = function handleRequest(
  props,
  state,
  elementRef,
  { url, key }
) {
  const fetchAndSetState = async () => {
    const fetch = await getFetch();
    const response = await fetch(url);
    const json = await response.json();
    const newOverrideState = { ...state.overrideState, [key]: json };
    state.overrideState = newOverrideState;
  };

  fetchAndSetState();
};
export const runHttpRequests = function runHttpRequests(
  props,
  state,
  elementRef
) {
  const requests =
    useContent(props, state, elementRef)?.data?.httpRequests ?? {};
  Object.entries(requests).forEach(([key, url]) => {
    if (url && (!httpReqsData(props, state, elementRef)[key] || isEditing())) {
      const evaluatedUrl = evalExpression(props, state, elementRef, url);
      handleRequest(props, state, elementRef, {
        url: evaluatedUrl,
        key,
      });
    }
  });
};
export const emitStateUpdate = function emitStateUpdate(
  props,
  state,
  elementRef
) {
  if (isEditing()) {
    window.dispatchEvent(
      new CustomEvent("builder:component:stateChange", {
        detail: {
          state: contentState(props, state, elementRef),
          ref: {
            name: props.model,
          },
        },
      })
    );
  }
};
export const shouldRenderContentStyles = function shouldRenderContentStyles(
  props,
  state,
  elementRef
) {
  return Boolean(
    (useContent(props, state, elementRef)?.data?.cssCode ||
      useContent(props, state, elementRef)?.data?.customFonts?.length) &&
      TARGET !== "reactNative"
  );
};
export const RenderContent = component$((props) => {
  const elementRef = useRef();
  const state = useStore({
    forceReRenderCount: 0,
    overrideContent: null,
    overrideState: {},
    update: 0,
  });
  useContextProvider(
    BuilderContext,
    useStore({
      content: (() => {
        return useContent(props, state, elementRef);
      })(),
      state: (() => {
        return contentState(props, state, elementRef);
      })(),
      context: (() => {
        return contextContext(props, state, elementRef);
      })(),
      apiKey: (() => {
        return props.apiKey;
      })(),
      registeredComponents: (() => {
        return allRegisteredComponents(props, state, elementRef);
      })(),
    })
  );
  useClientEffect$(() => {
    if (isBrowser()) {
      if (isEditing()) {
        state.forceReRenderCount = state.forceReRenderCount + 1;
        _useMutableProps(elementRef.current, true);

        registerInsertMenu();
        setupBrowserForEditing();
        Object.values(
          allRegisteredComponents(props, state, elementRef)
        ).forEach((registeredComponent) => {
          const message = createRegisterComponentMessage(registeredComponent);
          window.parent?.postMessage(message, "*");
        });
        window.addEventListener(
          "message",
          processMessage.bind(null, props, state, elementRef)
        );
        window.addEventListener(
          "builder:component:stateChangeListenerActivated",
          emitStateUpdate.bind(null, props, state, elementRef)
        );
      }

      if (useContent(props, state, elementRef)) {
        track({
          type: "impression",
          canTrack: canTrackToUse(props, state, elementRef),
          contentId: useContent(props, state, elementRef).id,
          orgId: props.apiKey,
        });
      } // override normal content in preview mode

      if (isPreviewing()) {
        const searchParams = new URL(location.href).searchParams;

        if (
          props.model &&
          searchParams.get("builder.preview") === props.model
        ) {
          const previewApiKey =
            searchParams.get("apiKey") || searchParams.get("builder.space");

          if (previewApiKey) {
            getContent({
              model: props.model,
              apiKey: previewApiKey,
            }).then((content) => {
              if (content) {
                state.overrideContent = content;
              }
            });
          }
        }
      }

      evaluateJsCode(props, state, elementRef);
      runHttpRequests(props, state, elementRef);
      emitStateUpdate(props, state, elementRef);
    }
  });
  useWatch$(({ track }) => {
    state.useContent?.data && track(state.useContent?.data, "jsCode");
    evaluateJsCode(props, state, elementRef);
  });
  useWatch$(({ track }) => {
    state.useContent?.data && track(state.useContent?.data, "httpRequests");
    runHttpRequests(props, state, elementRef);
  });
  useWatch$(({ track }) => {
    state && track(state, "contentState");
    emitStateUpdate(props, state, elementRef);
  });
  useCleanup$(() => {
    if (isBrowser()) {
      window.removeEventListener(
        "message",
        processMessage.bind(null, props, state, elementRef)
      );
      window.removeEventListener(
        "builder:component:stateChangeListenerActivated",
        emitStateUpdate.bind(null, props, state, elementRef)
      );
    }
  });
  return (
    <>
      {useContent(props, state, elementRef) ? (
        <div
          ref={elementRef}
          onClick$={(event) => onClick(props, state, elementRef, event)}
          builder-content-id={useContent(props, state, elementRef)?.id}
        >
          {shouldRenderContentStyles(props, state, elementRef) ? (
            <RenderContentStyles
              cssCode={useContent(props, state, elementRef)?.data?.cssCode}
              customFonts={
                useContent(props, state, elementRef)?.data?.customFonts
              }
            ></RenderContentStyles>
          ) : null}
          <RenderBlocks
            blocks={markMutable(
              useContent(props, state, elementRef)?.data?.blocks
            )}
            key={state.forceReRenderCount}
          ></RenderBlocks>
        </div>
      ) : null}
    </>
  );
});
export default RenderContent;
