import { isBrowser } from "../functions/is-browser";
import { getTopLevelDomain } from "./url";
const getCookie = ({
  name,
  canTrack
}) => {
  var _a;
  try {
    if (!canTrack) {
      return void 0;
    }
    return (_a = document.cookie.split("; ").find((row) => row.startsWith(`${name}=`))) == null ? void 0 : _a.split("=")[1];
  } catch (err) {
    console.debug("[COOKIE] GET error: ", err);
  }
};
const stringifyCookie = (cookie) => cookie.map(([key, value]) => value ? `${key}=${value}` : key).join("; ");
const SECURE_CONFIG = [
  ["secure", ""],
  ["SameSite", "None"]
];
const createCookieString = ({
  name,
  value,
  expires
}) => {
  const secure = isBrowser() ? location.protocol === "https:" : true;
  const secureObj = secure ? SECURE_CONFIG : [[]];
  const expiresObj = expires ? [["expires", expires.toUTCString()]] : [[]];
  const cookieValue = [
    [name, value],
    ...expiresObj,
    ["path", "/"],
    ["domain", getTopLevelDomain(window.location.hostname)],
    ...secureObj
  ];
  const cookie = stringifyCookie(cookieValue);
  return cookie;
};
const setCookie = ({
  name,
  value,
  expires,
  canTrack
}) => {
  try {
    if (!canTrack) {
      return void 0;
    }
    const cookie = createCookieString({ name, value, expires });
    document.cookie = cookie;
  } catch (err) {
    console.warn("[COOKIE] SET error: ", err);
  }
};
export {
  getCookie,
  setCookie
};
